#include "lowlevel.h"
#include "highlevel.h"
#include "highlevel_text.h"
#include "fonts.h"

// Bresenham Line Algorithm

void LCD_line(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2)
{
unsigned char x,y;
char dx,dy,incx,incy,balance;

        if (x2 >= x1)
        {
                dx = x2 - x1;
                incx = 1;
        }
        else
        {
                dx = x1 - x2;
                incx = -1;
        }

        if (y2 >= y1)
        {
                dy = y2 - y1;
                incy = 1;
        }
        else
        {
                dy = y1 - y2;
                incy = -1;
        }

        x = x1;
        y = y1;

        if (dx >= dy)
        {
                dy <<= 1;
                balance = dy - dx;
                dx <<= 1;

                while (x != x2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                y += incy;
                                balance -= dx;
                        }
                        balance += dy;
                        x += incx;
                } LCD_plot(x, y);
        }
        else
        {
                dx <<= 1;
                balance = dx - dy;
                dy <<= 1;

                while (y != y2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                x += incx;
                                balance -= dy;
                        }
                        balance += dx;
                        y += incy;
                } LCD_plot(x, y);
        }
}

void LCD_rectangle(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2)
{
	LCD_line(x1,y1,x2,y1);
	LCD_line(x2,y1,x2,y2);
	LCD_line(x2,y2,x1,y2);
	LCD_line(x1,y2,x1,y1);
}





void LCD_putchr6 ( unsigned char chr )
{
unsigned char i;
	for(i=0;i<5;i++)
		LCD_WriteData(font5x7[5*chr+i]);				// send character
	LCD_WriteData(0);										// send intercharacter separator	
}



void LCD_putchr8 ( unsigned char chr )
{
unsigned char i;
	for(i=0;i<8;i++)
		LCD_WriteData(font8x8[(chr<<3)+i]);				// send character
}

void LCD_putchrB ( unsigned char chr , unsigned char row, unsigned char col)
{
unsigned char i;

	for(i=0;i<8;i++)
		LCD_WriteData(font8x15[(chr<<4)+i]);			// send upper part
	LCD_WriteCmd (0xB9+row);							// point to next page address
	LCD_WriteCmd (0x40+col);							// restore original column
	for(;i<16;i++)
		LCD_WriteData(font8x15[(chr<<4)+i]);			// send lower part
}



void LCD_print6at (unsigned char row, unsigned char col, char *ptr)
{
unsigned char x;
	do {
		if(col>19){
			col=0;
			row++;
		}
		x=(col>9)?6*(col-10):6*col;				// correct x address if col>9
		if(col>9)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
			x+=4;											// 4 pixels offset at both sides
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address = 6*col or 6*col-10 (4 pixels offset)
		LCD_putchr6 (*ptr++);
		col++;
	} 	while (*ptr);
}

void LCD_print8at (unsigned char row, unsigned char col, char format, char *ptr)
{
unsigned char x;
	do {
		if(col>15){
			col=0;
			row++;
			if(format=='B')
				row++;
		}
		x=(col>7)?8*(col-8):8*col;					// correct x address if col>9
		if(col>7)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address
		switch(format){
			case 'B':
				LCD_putchrB (*ptr++,row,x);
				break;
			default:
				LCD_putchr8 (*ptr++);
				break;
		}
		col++;
	} 	while (*ptr);
}
